package org.jboss.qa.brms.fuse.remote;

import static org.ops4j.pax.exam.CoreOptions.maven;
import static org.ops4j.pax.exam.karaf.options.KarafDistributionOption.*;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.kie.api.runtime.manager.audit.AuditService;
import org.kie.api.runtime.manager.audit.ProcessInstanceLog;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.remote.client.api.RemoteRestRuntimeEngineBuilder;
import org.kie.remote.client.api.RemoteRestRuntimeEngineFactory;
import org.ops4j.pax.exam.Option;
import org.ops4j.pax.exam.junit.PaxExam;
import org.ops4j.pax.exam.karaf.options.KarafDistributionBaseConfigurationOption;
import org.ops4j.pax.exam.spi.reactors.ExamReactorStrategy;
import org.ops4j.pax.exam.spi.reactors.PerClass;

import static org.junit.Assert.assertNotNull;

/**
 * Created by jpetrlik on 1/27/16.
 */
@RunWith(PaxExam.class)
@ExamReactorStrategy(PerClass.class)
public class RemoteRestApiTest {

    /* Configuration */
    private static final String FRAMEWORK_URL = "file:/home/jpetrlik/bugs/remote-api-fuse-6.1-reproducer/jboss-fuse-6.1.1.zip"; // path to Fuse zip
    private static final String KARAF_VERSION = "2.3.0.redhat-611412"; // path to Fuse zip

    private static final String REMOTE_DEPLOYMENT_ID = "org.jboss.qa.brms.reproducer:rest-api-fuse-6.1.1-kjar:1.0-SNAPSHOT";
    private static final String REMOTE_USER = "user";
    private static final String REMOTE_PASSWD = "password1!";
    private static final String REMOTE_SERVER_URL = "http://localhost:8080/business-central/";

    /* DEPLOYED FEATURES */
    public static final String BXMS_FEATURE_GROUP_ID = "org.drools";
    public static final String BXMS_FEATURE_ARTIFACT_ID = "drools-karaf-features";

    public static final String KIE_REMOTE_CLIENT = "kie-remote-client";
    public static final String KIE_SERVER_CLIENT = "kie-server-client";

    private static String PROCESS_NAME = "DummyProcess";

    @org.ops4j.pax.exam.Configuration
    public Option[] config() throws IOException {
        KarafDistributionBaseConfigurationOption baseConfig = karafDistributionConfiguration()
                .frameworkUrl(FRAMEWORK_URL)
                .karafVersion(KARAF_VERSION);

        List<Option> opts = new ArrayList<Option>();
        opts.add(baseConfig);

        opts.add(features(maven().groupId(BXMS_FEATURE_GROUP_ID).artifactId(BXMS_FEATURE_ARTIFACT_ID)
                        .versionAsInProject().type("xml").classifier("features"),
                KIE_REMOTE_CLIENT, KIE_SERVER_CLIENT));

        return opts.toArray(new Option[opts.size()]);
    }

    @Test
    public void testRestRemoteApi() throws IOException {
        RuntimeEngine runtimeEngine = createRemoteRestRuntimeEngine();

        // run process and get process id
        KieSession kieSession = runtimeEngine.getKieSession();
        assertNotNull(kieSession);
        ProcessInstance processInstance = kieSession.startProcess(PROCESS_NAME);
        assertNotNull(processInstance);
        long processInstanceId = processInstance.getId();

        // find the process instance by id
        AuditService auditService = runtimeEngine.getAuditService();
        assertNotNull(auditService);
        ProcessInstanceLog processInstanceLog = auditService.findProcessInstance(processInstanceId);
        assertNotNull(processInstanceLog);
    }

    public static RuntimeEngine createRemoteRestRuntimeEngine() throws IOException {
        RemoteRestRuntimeEngineBuilder remoteRestRuntimeEngineBuilder = RemoteRestRuntimeEngineFactory.newBuilder();
        remoteRestRuntimeEngineBuilder.addUserName(REMOTE_USER);
        remoteRestRuntimeEngineBuilder.addPassword(REMOTE_PASSWD);
        remoteRestRuntimeEngineBuilder.addDeploymentId(REMOTE_DEPLOYMENT_ID);
        remoteRestRuntimeEngineBuilder.addUrl(new URL(REMOTE_SERVER_URL));

        RuntimeEngine runtimeEngine = remoteRestRuntimeEngineBuilder.buildFactory().newRuntimeEngine();
        return runtimeEngine;
    }
}
